about summary refs log tree commit diff
path: root/llvm_mode/afl-llvm-pass.so.cc
diff options
context:
space:
mode:
Diffstat (limited to 'llvm_mode/afl-llvm-pass.so.cc')
-rw-r--r--llvm_mode/afl-llvm-pass.so.cc91
1 files changed, 37 insertions, 54 deletions
diff --git a/llvm_mode/afl-llvm-pass.so.cc b/llvm_mode/afl-llvm-pass.so.cc
index 176692e3..cfeff968 100644
--- a/llvm_mode/afl-llvm-pass.so.cc
+++ b/llvm_mode/afl-llvm-pass.so.cc
@@ -118,7 +118,9 @@ bool AFLCoverage::runOnModule(Module &M) {
 
   }
 
-  char* neverZero_counters_str = getenv("AFL_NZERO_COUNTS");
+#if LLVM_VERSION_MAJOR < 9
+  char* neverZero_counters_str = getenv("AFL_LLVM_NOT_ZERO");
+#endif
 
   /* Get globals for the SHM region and the previous location. Note that
      __afl_prev_loc is thread-local. */
@@ -236,75 +238,56 @@ bool AFLCoverage::runOnModule(Module &M) {
       LoadInst *Counter = IRB.CreateLoad(MapPtrIdx);
       Counter->setMetadata(M.getMDKindID("nosanitize"), MDNode::get(C, None));
 
-      Value *Incr;
- //     if (neverZero_counters_str == NULL || neverZero_counters_str[0] != '4')
-        Incr = IRB.CreateAdd(Counter, ConstantInt::get(Int8Ty, 1));
-
-      if (neverZero_counters_str != NULL) {
-          /* hexcoder: Realize a counter that skips zero during overflow.
-           * Once this counter reaches its maximum value, it next increments to 1
-           *
-           * Instead of
-           * Counter + 1 -> Counter
-           * we inject now this
-           * Counter + 1 -> {Counter, OverflowFlag}
-           * Counter + OverflowFlag -> Counter
-           */
-           
-          // Solution #1 - creates
-          //mov    ecx,edx
-          //add    cl,0x1
-          //adc    dl,0x1
-/*
-          if (neverZero_counters_str[0] == '1') {
-            CallInst *AddOv = IRB.CreateBinaryIntrinsic(Intrinsic::uadd_with_overflow, Counter, ConstantInt::get(Int8Ty, 1));
-            AddOv->setMetadata(M.getMDKindID("nosanitize"), MDNode::get(C, None));
-            Value *SumWithOverflowBit = AddOv;
-            Incr = IRB.CreateAdd(IRB.CreateExtractValue(SumWithOverflowBit, 0),  // sum 
-                                 IRB.CreateZExt( // convert from one bit type to 8 bits type 
-                                                IRB.CreateExtractValue(SumWithOverflowBit, 1), // overflow
-                                                Int8Ty));
-
-          // Solution #2 - creates the same code as #1
+      Value *Incr = IRB.CreateAdd(Counter, ConstantInt::get(Int8Ty, 1));
+
+#if LLVM_VERSION_MAJOR < 9
+      if (neverZero_counters_str != NULL) { // with llvm 9 we make this the default as the bug in llvm is then fixed
+#endif
+        /* hexcoder: Realize a counter that skips zero during overflow.
+         * Once this counter reaches its maximum value, it next increments to 1
+         *
+         * Instead of
+         * Counter + 1 -> Counter
+         * we inject now this
+         * Counter + 1 -> {Counter, OverflowFlag}
+         * Counter + OverflowFlag -> Counter
+         */
+/*       // we keep the old solutions just in case
+         // Solution #1
+         if (neverZero_counters_str[0] == '1') {
+           CallInst *AddOv = IRB.CreateBinaryIntrinsic(Intrinsic::uadd_with_overflow, Counter, ConstantInt::get(Int8Ty, 1));
+           AddOv->setMetadata(M.getMDKindID("nosanitize"), MDNode::get(C, None));
+           Value *SumWithOverflowBit = AddOv;
+           Incr = IRB.CreateAdd(IRB.CreateExtractValue(SumWithOverflowBit, 0),  // sum 
+                                IRB.CreateZExt( // convert from one bit type to 8 bits type 
+                                               IRB.CreateExtractValue(SumWithOverflowBit, 1), // overflow
+                                               Int8Ty));
+          // Solution #2
           } else if (neverZero_counters_str[0] == '2') {
-            auto cf = IRB.CreateICmpULT(Incr, ConstantInt::get(Int8Ty, 1));
-            Incr = IRB.CreateAdd(Incr, cf);
-           
-          // Solution #3 - creates
-          //add    cl,0x1
-          //cmp    cl,0x1
-          //adc    cl,0x0
-          } else if (neverZero_counters_str[0] == '3') {
-            auto cf = IRB.CreateICmpEQ(Incr, ConstantInt::get(Int8Ty, 0));
-            Incr = IRB.CreateAdd(Incr, cf);
-          // Solution #4 - creates
-          // cmp    dl, $0xff
-          // sete   cl
-          // add    dl,cl
-          // add    dl,0x01
-          } else if (neverZero_counters_str[0] == '4') {
              auto cf = IRB.CreateICmpEQ(Counter, ConstantInt::get(Int8Ty, 255));
              Value *HowMuch = IRB.CreateAdd(ConstantInt::get(Int8Ty, 1), cf);
              Incr = IRB.CreateAdd(Counter, HowMuch);
-
-          } else if (neverZero_counters_str[0] == '5') {
+          // Solution #3
+          } else if (neverZero_counters_str[0] == '3') {
 */
+          // this is the solution we choose because llvm9 should do the right thing here
             auto cf = IRB.CreateICmpEQ(Incr, ConstantInt::get(Int8Ty, 0));
             auto carry = IRB.CreateZExt(cf, Int8Ty);
             Incr = IRB.CreateAdd(Incr, carry);
 /*
-          } else if (neverZero_counters_str[0] == '6') {
+         // Solution #4
+         } else if (neverZero_counters_str[0] == '4') {
             auto cf = IRB.CreateICmpULT(Incr, ConstantInt::get(Int8Ty, 1));
             auto carry = IRB.CreateZExt(cf, Int8Ty);
             Incr = IRB.CreateAdd(Incr, carry);
-           
-          // no other implementations yet
-          } else {
+         } else {
             fprintf(stderr, "Error: unknown value for AFL_NZERO_COUNTS: %s (valid is 1-4)\n", neverZero_counters_str);
             exit(-1);
-          }
+         }
 */
+#if LLVM_VERSION_MAJOR < 9
       }
+#endif
 
       IRB.CreateStore(Incr, MapPtrIdx)->setMetadata(M.getMDKindID("nosanitize"), MDNode::get(C, None));